package javax.swing;

import java.awt.event.ItemEvent;
import java.io.Serializable;
import java.awt.event.ItemListener;
import java.awt.event.ActionListener;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.EventObject;
import java.beans.ConstructorProperties;
import javax.swing.tree.TreeCellEditor;
import javax.swing.table.TableCellEditor;

public class DefaultCellEditor extends AbstractCellEditor implements TableCellEditor, TreeCellEditor
{
    protected JComponent editorComponent;
    protected DefaultCellEditor.EditorDelegate delegate;
    protected int clickCountToStart;
    
    @ConstructorProperties({ "component" })
    public DefaultCellEditor(final JTextField editorComponent) {
        this.clickCountToStart = 1;
        this.editorComponent = editorComponent;
        this.clickCountToStart = 2;
        editorComponent.addActionListener(this.delegate = new DefaultCellEditor.EditorDelegate() {
            @Override
            public void setValue(Object o) {
                editorComponent.setText((o != null) ? o.toString() : "");
            }
            
            @Override
            public Object getCellEditorValue() {
                return editorComponent.getText();
            }
        });
    }
    
    public DefaultCellEditor(final JCheckBox editorComponent) {
        this.clickCountToStart = 1;
        (this.editorComponent = editorComponent).addActionListener(this.delegate = new DefaultCellEditor.EditorDelegate() {
            @Override
            public void setValue(Object o) {
                boolean selected = false;
                if ((o instanceof Boolean)) {
                    selected = (boolean)o;
                }
                else if ((o instanceof String)) {
                    selected = o.equals("true");
                }
                editorComponent.setSelected((boolean)selected);
            }
            
            @Override
            public Object getCellEditorValue() {
                return Boolean.valueOf(editorComponent.isSelected());
            }
        });
        editorComponent.setRequestFocusEnabled(false);
    }
    
    public DefaultCellEditor(final JComboBox editorComponent) {
        this.clickCountToStart = 1;
        (this.editorComponent = editorComponent).putClientProperty("JComboBox.isTableCellEditor", Boolean.TRUE);
        editorComponent.addActionListener(this.delegate = new DefaultCellEditor.EditorDelegate() {
            @Override
            public void setValue(Object selectedItem) {
                editorComponent.setSelectedItem(selectedItem);
            }
            
            @Override
            public Object getCellEditorValue() {
                return editorComponent.getSelectedItem();
            }
            
            @Override
            public boolean shouldSelectCell(EventObject eventObject) {
                return !(eventObject instanceof MouseEvent) || ((MouseEvent)eventObject).getID() != 506;
            }
            
            @Override
            public boolean stopCellEditing() {
                if (editorComponent.isEditable()) {
                    editorComponent.actionPerformed(new ActionEvent(DefaultCellEditor.this, 0, ""));
                }
                return super.stopCellEditing();
            }
        });
    }
    
    public Component getComponent() {
        return this.editorComponent;
    }
    
    public void setClickCountToStart(int clickCountToStart) {
        this.clickCountToStart = clickCountToStart;
    }
    
    public int getClickCountToStart() {
        return this.clickCountToStart;
    }
    
    @Override
    public Object getCellEditorValue() {
        return this.delegate.getCellEditorValue();
    }
    
    @Override
    public boolean isCellEditable(EventObject eventObject) {
        return this.delegate.isCellEditable(eventObject);
    }
    
    @Override
    public boolean shouldSelectCell(EventObject eventObject) {
        return this.delegate.shouldSelectCell(eventObject);
    }
    
    @Override
    public boolean stopCellEditing() {
        return this.delegate.stopCellEditing();
    }
    
    @Override
    public void cancelCellEditing() {
        this.delegate.cancelCellEditing();
    }
    
    @Override
    public Component getTreeCellEditorComponent(JTree tree, Object o, boolean b, boolean b2, boolean b3, int n) {
        this.delegate.setValue(tree.convertValueToText(o, b, b2, b3, n, false));
        return this.editorComponent;
    }
    
    @Override
    public Component getTableCellEditorComponent(JTable table, Object value, boolean b, int n, int n2) {
        this.delegate.setValue(value);
        if ((this.editorComponent instanceof JCheckBox)) {
            Component tableCellRendererComponent = table.getCellRenderer(n, n2).getTableCellRendererComponent(table, value, b, true, n, n2);
            if (tableCellRendererComponent != null) {
                this.editorComponent.setOpaque(true);
                this.editorComponent.setBackground(tableCellRendererComponent.getBackground());
                if ((tableCellRendererComponent instanceof JComponent)) {
                    this.editorComponent.setBorder(((JComponent)tableCellRendererComponent).getBorder());
                }
            }
            else {
                this.editorComponent.setOpaque(false);
            }
        }
        return this.editorComponent;
    }
    
    protected class EditorDelegate implements ActionListener, ItemListener, Serializable
    {
        protected Object value;
        
        public Object getCellEditorValue() {
            return this.value;
        }
        
        public void setValue(Object value) {
            this.value = value;
        }
        
        public boolean isCellEditable(EventObject eventObject) {
            return !(eventObject instanceof MouseEvent) || ((MouseEvent)eventObject).getClickCount() >= DefaultCellEditor.this.clickCountToStart;
        }
        
        public boolean shouldSelectCell(EventObject eventObject) {
            return true;
        }
        
        public boolean startCellEditing(EventObject eventObject) {
            return true;
        }
        
        public boolean stopCellEditing() {
            DefaultCellEditor.this.fireEditingStopped();
            return true;
        }
        
        public void cancelCellEditing() {
            DefaultCellEditor.this.fireEditingCanceled();
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            DefaultCellEditor.this.stopCellEditing();
        }
        
        @Override
        public void itemStateChanged(ItemEvent itemEvent) {
            DefaultCellEditor.this.stopCellEditing();
        }
    }
}
